package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamDetailCourseDto;
import com.qmth.distributed.print.business.bean.dto.SyncExamTaskDto;
import com.qmth.distributed.print.business.entity.ExamCard;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.service.ExamCardService;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.enums.CardCreateMethodEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 考务数据（考场关联科目） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailCourseServiceImpl extends ServiceImpl<ExamDetailCourseMapper, ExamDetailCourse> implements ExamDetailCourseService {

    @Resource
    private ExamTaskDetailService examTaskDetailService;

    @Resource
    private ExamCardService examCardService;

    @Override
    public List<Map<String, Object>> listByExamDetailId(Long examDetailId) {
        return this.baseMapper.listByExamDetailId(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public void updatePaperNumber(List<Long> examDetailCourseIds, String paperNumber, String relatePaperType) {
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumber)
                .set(ExamDetailCourse::getPaperType, relatePaperType)
                .in(ExamDetailCourse::getId, examDetailCourseIds);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listByExamDetailIdAndStatus(examDetailId, ExamStatusEnum.SUBMIT.name());
    }

    @Override
    public List<BasicCourse> listCoursesByPrintPlanId(String param, List<Long> printPlanIdList) {
        return this.baseMapper.listCoursesByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList);
    }

    @Override
    public List<String> listPaperNumberByPrintPlanId(String param, List<Long> printPlanIdList, Set<Long> orgIds) {
        return this.baseMapper.listPaperNumberByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList, orgIds);
    }

    @Override
    public List<ExamDetailCourse> listByPrintPlanIdAndCourseCodeAndPaperNumber(Long schoolId, Long printPlanId, String courseCode, String paperNumber) {
        return this.baseMapper.listByPrintPlanIdAndCourseCodeAndPaperNumber(schoolId, printPlanId, courseCode, paperNumber);
    }

    @Override
    public List<ExamDetailCourse> listByCourseCodeAndPaperNumber(Long schoolId, String courseCode, String paperNumber) {
        return this.baseMapper.listByAndCourseCodeAndPaperNumber(schoolId, courseCode, paperNumber);
    }

    @Override
    public List<SyncExamTaskDto> listSyncCourseByPrintPlanId(List<Long> printPlanIds, SyncCardTypeEnum cardType, String courseName) {
        List<SyncExamTaskDto> examTaskDtos = this.baseMapper.listSyncCourseByPrintPlanId(printPlanIds, courseName);
        for (SyncExamTaskDto examTaskDto : examTaskDtos) {
            String paperType = examTaskDto.getPaperType();
            List<String> paperTypes = Arrays.asList(paperType.split(","));
            ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamTaskId(examTaskDto.getId());
            List<JSONObject> paperAttachments = JSON.parseArray(examTaskDetail.getPaperAttachmentIds(), JSONObject.class);
            List<Long> cardIds = paperAttachments.stream().filter(m -> paperTypes.contains(m.getString("name"))).map(m -> m.getLong("cardId")).collect(Collectors.toList());
            QueryWrapper<ExamCard> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(ExamCard::getId, cardIds);

            List<ExamCard> cardList = examCardService.list(queryWrapper);
            Set<String> stringSet = new HashSet<>();
            for (ExamCard examCard : cardList) {
                if (examCard.getType().equals(CardTypeEnum.GENERIC) && examCard.getCreateMethod().equals(CardCreateMethodEnum.UPLOAD)) {
                    stringSet.add(SyncCardTypeEnum.GENERIC.getDesc());
                } else {
                    stringSet.add(SyncCardTypeEnum.CUSTOM.getDesc());
                }
            }
            if (stringSet.size() > 0) {
                examTaskDto.setSyncCardType(String.join(",", stringSet));
            }

            // 解析同步状态
            String syncStatus = examTaskDto.getSyncStatus();
            if (StringUtils.isNotBlank(syncStatus)) {
                List<String> collect = Stream.of(syncStatus.split(",")).map(m -> ExamPrintPlanSyncStatusEnum.valueOf(m).getDesc()).collect(Collectors.toList());
                examTaskDto.setSyncStatus(collect.isEmpty() ? "-" : String.join(",", collect));
            }
        }
        if(cardType != null){
            examTaskDtos = examTaskDtos.stream().filter(m->m.getSyncCardType().contains(cardType.getDesc())).collect(Collectors.toList());
        }
        return examTaskDtos;
    }

    @Override
    public List<ExamDetailCourseDto> listByPrintPlanIdAndExamTaskId(List<Long> printPlanIds, Long examTaskId) {
        return this.baseMapper.listByPrintPlanIdAndExamTaskId(printPlanIds, examTaskId);
    }
}
