package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.TSyncStmmsExam;
import com.qmth.distributed.print.business.mapper.TSyncStmmsExamMapper;
import com.qmth.distributed.print.business.service.TSyncStmmsExamService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.sync.CloudMarkingTaskUtils;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class TSyncStmmsExamServiceImpl extends ServiceImpl<TSyncStmmsExamMapper, TSyncStmmsExam> implements TSyncStmmsExamService {

    @Resource
    CloudMarkingTaskUtils cloudMarkingTaskUtils;

    @Override
    public List<TSyncStmmsExam> listExam() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<TSyncStmmsExam> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TSyncStmmsExam::getSchoolId, schoolId)
                .orderByDesc(TSyncStmmsExam::getExamId);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public List<TSyncStmmsExam> syncExam() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        UpdateWrapper<TSyncStmmsExam> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(TSyncStmmsExam::getSchoolId, schoolId);
        this.remove(updateWrapper);

        List<JSONObject> objectList = cloudMarkingTaskUtils.getExam(schoolId, SystemConstant.PAGE_NUMBER_MIN, SystemConstant.PAGE_SIZE);
        if (objectList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("未查询到考试");
        }
        List<TSyncStmmsExam> markingExams = objectList.stream().map(m -> {
            TSyncStmmsExam cloudMarkingExam = new TSyncStmmsExam();
            cloudMarkingExam.setId(SystemConstant.getDbUuid());
            cloudMarkingExam.setSchoolId(schoolId);
            cloudMarkingExam.setExamId(m.getInteger("id"));
            cloudMarkingExam.setExamName(m.getString("name"));
            return cloudMarkingExam;
        }).collect(Collectors.toList());
        if (!markingExams.isEmpty()) {
            this.saveBatch(markingExams);
        }
        return markingExams;
    }

    @Override
    public TSyncStmmsExam getBySchoolIdAndExamId(Long schoolId, Long thirdRelateId) {
        QueryWrapper<TSyncStmmsExam> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TSyncStmmsExam::getSchoolId, schoolId)
                .eq(TSyncStmmsExam::getExamId, thirdRelateId);
        return this.getOne(queryWrapper);
    }
}
