package com.qmth.distributed.print.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.params.SyncDataParam;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.enums.SyncCardTypeEnum;
import com.qmth.distributed.print.business.service.ExamPrintPlanService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * <p>
 * 印刷计划推送管理 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "印刷计划推送管理Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_EXAM + "/print_sync")
@Validated
public class ExamPrintPlanSyncController {
    @Resource
    private ExamPrintPlanService examPrintPlanService;

    /**
     * 需要合并的计划查询
     *
     * @param printPlanId
     * @param startTime
     * @param endTime
     * @param pageNumber
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "印刷计划推送管理-查询")
    @RequestMapping(value = "/list_sync", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    public Result findPrintPlanPage(@ApiParam(value = "学期ID") @RequestParam(required = false) Long semesterId,
                                    @ApiParam(value = "考试ID") @RequestParam(required = false) Long examId,
                                    @ApiParam(value = "印刷计划ID") @RequestParam(required = false) Long printPlanId,
                                    @ApiParam(value = "计划创建时间段开始时间") @RequestParam(required = false) Long startTime,
                                    @ApiParam(value = "计划创建时间段结束时间") @RequestParam(required = false) Long endTime,
                                    @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                    @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return ResultUtil.ok(examPrintPlanService.printPlanSyncPage(schoolId, semesterId, examId, printPlanId, startTime, endTime, pageNumber, pageSize));
    }

    /**
     * 查询计划下需要同步的命题任务
     *
     * @param printPlanIds 印刷计划ID
     */
    @ApiOperation(value = "印刷计划推送管理-推送数据查询")
    @RequestMapping(value = "/list_sync_exam_task", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    public Result listSyncExamTask(@ApiParam(value = "印刷计划ID集合") @RequestParam List<Long> printPlanIds,
                                   @ApiParam(value = "题卡类型") @RequestParam(required = false) SyncCardTypeEnum cardType,
                                   @ApiParam(value = "课程名称") @RequestParam(required = false) String courseName) {
        return ResultUtil.ok(examPrintPlanService.listSyncExamTask(printPlanIds, cardType, courseName));
    }


    /**
     * 多计划合并同步
     *
     * @param syncDataParam
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "印刷计划推送管理-开始推送")
    @RequestMapping(value = "/sync_data_cloud", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.PUSH)
    public Result syncDataCloud(@RequestBody SyncDataParam syncDataParam) {
        examPrintPlanService.syncDataCloud(syncDataParam);
        return ResultUtil.ok(true);
    }

}

