SET NAMES utf8mb4;

DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint NOT NULL,
                                 `name` varchar(100) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件',
                                 `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int NOT NULL COMMENT '序号',
                                 `property` varchar(30) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 UNIQUE KEY `privilege_index` (`name`,`url`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

INSERT INTO `sys_privilege` VALUES (1, '基础配置', 'base', 'MENU', NULL, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (77, '用户管理-查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (78, '用户管理-新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (79, '用户管理-启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (80, '用户管理-重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (81, '用户管理-导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (82, '用户管理-绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (83, '角色管理-查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (84, '角色管理-删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (85, '角色管理-新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (86, '角色管理-启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (87, '用户管理-用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (88, '权限管理-查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (89, '权限管理-新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (90, '权限管理-删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (91, '角色管理-角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (129, '公共接口', 'common', 'MENU', NULL, 31, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (180, '角色管理-查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (184, '用户管理-查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (250, '教研分析', 'report', 'MENU', NULL, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (251, '学期列表', '/api/report/school/common/list_semester', 'URL', 250, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (252, '考试列表', '/api/report/school/common/list_exam', 'URL', 250, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (253, '课程列表', '/api/report/school/common/list_course', 'URL', 250, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (254, '开课学院列表', '/api/report/school/common/list_teach_college', 'URL', 250, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (255, '考查学院列表', '/api/report/school/common/list_inspect_college', 'URL', 250, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (256, '教师列表', '/api/report/school/common/list_teacher', 'URL', 250, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (257, '班级列表', '/api/report/school/common/list_class', 'URL', 250, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (258, '数据驾驶舱', 'cockpit', 'MENU', 250, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (259, '全校考试总览', 'SchoolExam', 'MENU', 258, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (260, '全校考试概况', '/api/report/school/exam/get_overview', 'URL', 259, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (261, '全校课程考试分析汇总', '/api/report/school/exam/get_course_summary', 'URL', 259, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (262, '全校课程考试分析汇总-导出课程分析Excel报表', '/api/report/school/exam/export_course_summary', 'URL', 259, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (263, '全校课程考试分析', 'SchoolCourse', 'MENU', 258, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (264, '考试概况', '/api/report/school/course/get_overview', 'URL', 263, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (265, '分数段分布情况', '/api/report/school/course/get_grade_distribute', 'URL', 263, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (266, '教师各课堂成绩排名', '/api/report/school/course/list_teacher_class_rank', 'URL', 263, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (267, '导出课程分析报表', '/api/report/school/course/export_data_analysis', 'URL', 263, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (268, '全校教师各课堂成绩排名', 'SchoolTeacherClassScore', 'MENU', 258, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (269, '查询', '/api/report/school/teacher/list_all_teacher_class_rank', 'URL', 268, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (270, '学生', 'student', 'MENU', 250, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (271, '开课成绩查询', 'TeachCollegeScore', 'MENU', 270, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (272, '查询', '/api/report/school/student/list_teach_college_result', 'URL', 271, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (273, '导入异常考试类型', '/api/report/school/student/import_abnormal_data', 'URL', 271, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (274, '导出查询结果', '/api/report/school/student/export_teach_college_result', 'URL', 271, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (275, '考查成绩查询', 'InspectCollegeScore', 'MENU', 270, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (276, '查询', '/api/report/school/student/list_inspect_college_result', 'URL', 275, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (277, '导出查询结果', '/api/report/school/student/export_inspect_college_result', 'URL', 275, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (278, '教师', 'teacher', 'MENU', 250, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (279, '教师各课堂成绩排名', 'TeacherClassScore', 'MENU', 278, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (280, '查询', '/api/report/school/teacher/list_class_rank', 'URL', 279, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (281, '我的课堂', 'MyClassScore', 'MENU', 278, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (282, '查询', '/api/report/school/teacher/list_myclass_rank', 'URL', 281, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (283, '课程', 'course', 'MENU', 250, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (284, '考查课程考试总览', 'InspectCourseExamOverview', 'MENU', 283, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (285, '考查课程考试分析', 'InspectCourseExamAnalysis', 'MENU', 283, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (286, '开课课程考试总览', 'OwnerCourseExamOverview', 'MENU', 283, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (287, '开课课程考试分析', 'OwnerCourseExamAnalysis', 'MENU', 283, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (288, '赋分管理', 'AssignScoreAction', 'MENU', 283, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (289, '考查课程考试总览-公共课概况接口', '/api/report/course/survey_inspect/view', 'URL', 284, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (290, '考查课程考试分析-图表接口', '/api/report/course/survey_inspect_analyse/view', 'URL', 285, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (291, '开课课程考试总览-列表接口', '/api/report/course/survey_teacher/list', 'URL', 286, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (292, '开课课程考试总览-图表接口', '/api/report/course/survey_teacher/view', 'URL', 287, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (293, '赋分管理-列表接口', '/api/report/course/survey_aspoints/list', 'URL', 288, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (294, '赋分管理-开始&重新试算接口', '/api/report/course/survey_aspoints/calculation', 'URL', 288, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (295, '赋分管理-预览&查看接口', '/api/report/course/survey_aspoints/view', 'URL', 288, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (296, '赋分管理-发布&撤回接口', '/api/report/course/survey_aspoints/publish', 'URL', 288, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (297, '赋分管理-保存&保存并发布接口', '/api/report/course/survey_aspoints/save', 'URL', 288, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (298, '学生报告预览', 'StudentReportManage', 'MENU', 270, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (299, '学生报告预览-列表接口', '/api/report/exam_student/report/list', 'URL', 298, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (300, '学生报告-个人成绩总览接口', '/api/report/exam_student/report/result', 'URL', 298, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (301, '学生报告接口', '/api/report/exam_student/report/mark', 'URL', 298, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (302, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (303, '开课课程考试总览-教师各课堂成绩排名接口', '/api/report/course/survey_teacher/teacher_view', 'URL', 287, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (304, '开课课程考试总览-试题难度得分情况分析接口', '/api/report/course/survey_teacher/question_view', 'URL', 287, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (305, '开课课程考试总览-各难度水平题目上的作答分析接口', '/api/report/course/survey_teacher/question_answer_view', 'URL', 287, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (306, '常规配置信息导入：试卷难易度和学院排名百分等级档位', '/api/report/datasource/config/import', 'URL', 250, 100, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (307, '知识点导入模板', '/api/report/datasource/dimension/import', 'URL', 250, 101, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (308, '考试作答信息导入模板', '/api/report/datasource/examAnswer/import', 'URL', 250, 102, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (309, '模块和等级导入接口', '/api/report/datasource/moduleLevel/import', 'URL', 250, 103, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (310, '公式赋分操作', '/api/report/datasource/normal/assignScore', 'URL', 250, 104, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (311, '试卷数据导入', '/api/report/datasource/paper/import', 'URL', 250, 105, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (312, '特殊科目赋分操作', '/api/report/datasource/special/assignScore', 'URL', 250, 106, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (313, '数据计算', '/api/report/analyze/calculate', 'URL', 250, 107, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (314, '任课老师-课程列表', '/api/report/school/common/list_course_teach', 'URL', 250, 8, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (315, '开课课程考试总览-列表导出', '/api/report/course/survey_teacher/export', 'URL', 286, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (316, '考务数据同步', '/api/report/datasource/examination/sync', 'URL', 250, 109, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (317, '创建课程', '/api/report/datasource/course/create', 'URL', 250, 108, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (318, '公共实际（应届不缺考）学院查询', '/api/report/school/common/list_inspect_college_not_absent', 'URL', 250, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (319, '公共实际（应届不缺考）教师查询', '/api/report/school/common/list_teacher_not_absent', 'URL', 250, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (320, '导出课程分析接口', '/api/report/course/survey_teacher/course_export', 'URL', 250, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (321, '武大学生开放鉴权接口', '/api/report/wuda/open/authentication', 'URL', 250, 11, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (322, '异常考生-批量处理', '/api/report/datasource/exception_student/batch_dispose', 'URL', 250, 110, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (323, '特殊科目合并题目处理', '/api/report/datasource/special/combine', 'URL', 250, 111, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (324, '异常答案/分数处理（先处理异常后赋分）', '/api/report/datasource/exception_answer/score_dispose', 'URL', 250, 112, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (325, '武大学生开放鉴权用户临时授权登录接口', '/api/report/wuda/open/user/authentication/login', 'URL', 250, 14, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (326, '武大学生开放鉴权测试接口', '/api/report/wuda/open/user/authentication/test', 'URL', 250, 15, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (327, '武大用户鉴权退出接口', '/api/report/wuda/open/user/authentication/logout', 'URL', 250, 16, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (328, '留学生异常处理', '/api/report/datasource/exception_student/dispose', 'URL', 250, 113, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (329, '异常考生-按学号处理', '/api/report/datasource/exception_student/dispose_by_student_code', 'URL', 250, 17, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (330, '我的课堂考试分析', 'MyClassExamView', 'MENU', 278, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (331, '我的课堂考试分析-图表接口', '/api/report/school/teacher/view', 'URL', 330, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (332, '我的课堂考试分析-试题难度得分情况分析接口', '/api/report/school/teacher/question_view', 'URL', 330, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (333, '我的课堂考试分析-各难度水平题目上的作答分析接口', '/api/report/school/teacher/question_answer_view', 'URL', 330, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (334, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (335, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 334, 1, 'AUTH', '336,337,338', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (336, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 335, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (337, '离线激活', '/api/admin/auth/offline/activation', 'URL', 335, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (338, '查询激活信息', '/api/admin/auth/select', 'URL', 335, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (339, '学校管理', 'SchoolManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (340, '查询', 'Select', 'BUTTON', 339, 1, 'AUTH', '342', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (341, '查询条件', 'Condition', 'CONDITION', 339, 2, 'AUTH', '342', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (342, '查询', '/api/admin/commom/school/list', 'URL', 339, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (343, '机构同步', 'Sync', 'BUTTON', 339, 3, 'AUTH', '344', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (344, '机构同步', '/api/admin/auth/sync/org', 'URL', 339, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (345, '编辑', 'Edit', 'LINK', 339, 1, 'AUTH', '346', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (346, '编辑', '/api/admin/commom/school/update', 'URL', 339, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (347, '同步配置保存', '/api/report/set/sync/save', 'URL', 339, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (348, '同步配置查询', '/api/report/set/sync/select', 'URL', 339, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (349, '设置', 'Set', 'LINK', 339, 2, 'AUTH', '347,348', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (138647380756004864, '赋分即时预览', 'AssignScoreActionTry', 'MENU', 283, 6, NULL, NULL, 1, 1, 1);

INSERT INTO `sys_config` VALUES (9, NULL, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (10, NULL, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (11, NULL, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (12, NULL, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (13, NULL, NULL, 'auto.create.pdf.reset.max.count', '自动重试生成pdf失败最大次数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (14, NULL, NULL, 'thread.pool.core.size', '自定义线程池大小', '1', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (15, NULL, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (16, NULL, NULL, 'session.active', 'session会话时长', '2h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (18, NULL, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (19, NULL, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (20, NULL, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (21, NULL, NULL, 'teachcloud.report.student.url', '学生报告地址', 'http://localhost:9099/#/student-report/', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (22, NULL, NULL, 'teachcloud.report.login.url', '单点登录地址', 'http://127.0.0.1:8057/#/login-authen/', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (24, NULL, NULL, 'teachcloud.print.open.callback.pwd', '分布式印刷回调密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);

delete t from `sys_config` t where t.id = 6;
